var ImportSpecimen = function()
{
	var specimen = {};
	var debug = 0;
	var importItems;

	var autoAnalyzeIsValid = false;

	specimen.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', specimen.validateStep1);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', specimen.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', specimen.validateStep2);
		Events.addEventHandler('clusters', 'change', specimen.clusterOnChange);
		Events.addEventHandler('autoAnalyzeYes', 'click', specimen.autoAnalyzeOnChange);
		Events.addEventHandler('autoAnalyzeNo', 'click', specimen.autoAnalyzeOnChange);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('goregister', Wizard.goRegister);

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', specimen.submit);

		Wizard.showLoadingAnimation('Loading items...');
		var url = '../ExternalSpecimen.servlet?ID='+App.getSessionId();
		url += '&cmd=GetImportInformation';
		Wizard.asyncJsonRequest(url, specimen.initializeStep1);
	}

	// --- Step 1 -----------------------------------
	specimen.initializeStep1 = function(response)
	{
		importItems = response.importItems;
		if (importItems.length == 0)
		{
			Wizard.setFatalError('No items available for importing.');
			return;
		}
		
		importItems.sort(specimen.sortByFileDate);
		
		var html = '<table id="importItemsTable">';
		html += '<thead class="bg-filled-100">';
		// Header row
		
		html += '<tr>';
		html += '<th>##</th>';
		html += '<th class="dottedleft">Item</th>';
		html += '<th class="dottedleft">Case</th>';
		html += '<th class="dottedleft">JSON file</th>'
		html += '<th>Date</th>';
		html += '<th>Reads</th>';
		html += '<th>FASTQ files</th>';
		html += '<th class="icon"></th>'
		html += '<th class="valid dottedleft" colspan="2">Import</th>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';

		for (var itemNo = 0; itemNo < importItems.length; itemNo++)
		{
			var item = importItems[itemNo];
			var jsonFile = item.jsonFile;
			var canImport = item.name && jsonFile && !jsonFile.errors;
			
			var icon = null;
			if (jsonFile)
			{
				icon = 'ok.png';
				if (jsonFile.warnings) icon = 'warning.png';
				if (jsonFile.errors) icon = 'error.png';
			}

			html += '<tr class="highlight ' + (itemNo % 4 < 2 ? 'evenrow' : 'oddrow') + '">';
			html += '<th>'+(itemNo+1)+'</th>';
			html += '<td class="dottedleft">';
			if (item.name)
			{
				html += Strings.encodeTags(item.name);
				if (!item.id) html += ' <img src="../images/new.png" title="This item will be created by the importer">';
				if (item.itemType) html += ' <span class="itemsubtype">('+Strings.encodeTags(item.itemType)+')</span>';
			}
			else
			{
				html += '-';
			}
			html += '<td class="dottedleft">';
			if (item.currentCase)
			{
				html += Strings.encodeTags(item.currentCase.name);
				html += '<img src="../images/case_summary.png" class="link case-summary" data-name="'+Strings.encodeTags(item.currentCase.name)+'" title="Show case summary (popup window)">';
			}
			else
			{
				html += '-';
			}
			html += '</td>';
			if (jsonFile)
			{
				html += '<td class="dottedleft" style="white-space: nowrap;">';
				html += Strings.encodeTags(jsonFile.name);
				html += ' <span class="file-data">('+Numbers.formatBytes(jsonFile.size)+')</span>';
				html += ' <span class="link download-file" data-file-no="'+itemNo+'" title="Download and view this file"><img src="../images/download.png"></span>';
				html += '</td>';
				html += '<td style="white-space: nowrap;">';
				html += Reggie.reformatDate(jsonFile.lastModified);
				html += '</td>';
				html += '<td>';
				html += Reggie.formatCount(jsonFile.PF_READS);
				html += '</td>';
				html += '<td class="fastq" style="white-space: nowrap;">';
				if (jsonFile && jsonFile.fastq)
				{
					for (var fqNo = 0; fqNo < jsonFile.fastq.length; fqNo++)
					{
						var fq = jsonFile.fastq[fqNo];
						html += Strings.encodeTags(fq.name);
						html += ' <span class="file-data">('
						if (fq.size > 0) 
						{
							html += Numbers.formatBytes(fq.size)+'; '+Reggie.reformatDate(fq.lastModified);
						}
						else
						{
							html += 'missing';
						}
						html += ')</span><br>';
					}
				}
				html += '</td>';
			}
			else
			{
				html += '<td class="dottedleft" colspan="4" style="text-align: left;">';
				if (item.externalSpecimenExists)
				{
					html += '<b>Specimen exists:</b> '+Strings.encodeTags(item.externalSpecimenExists);
					if (item.externalOperator) html += '/'+Strings.encodeTags(item.externalOperator);
				}
				if (item.samplingDate)
				{
					html += '; <b>Sampling date:</b> '+Reggie.reformatDate(item.samplingDate);
				}
				if (item.registered)
				{
					html += '; <b>Registration date:</b> '+Reggie.reformatDate(item.registered);
				}
				html += '</td>';
			}
			html += '<td class="icon">';
			if (icon) html+= '<img src="../images/'+icon+'">';
			html += '</td>';
			html += '<td class="import dottedleft">';
			if (canImport)
			{
				html += '<input type="checkbox" id="import.'+itemNo+'" checked>';
			}
			html += '</td>';
			html += '<td class="valid">';
			var startNo = 1;
			if (jsonFile && jsonFile.errors)
			{
				html += '<div class="messagecontainer error"><ol start="'+startNo+'">';
				for (var errNo = 0; errNo < jsonFile.errors.length; errNo++)
				{
					var msg = jsonFile.errors[errNo];
					if (msg.indexOf('[HTML]')==0)
					{
						html += '<li>'+msg.replace('[HTML]', '');
					}
					else
					{
						html += '<li>'+Strings.encodeTags(msg);
					}
				}
				html += '</ol></div>';
				startNo += jsonFile.errors.length;
			}
			if (jsonFile && jsonFile.warnings)
			{
				var moreWarnings = jsonFile.warnings.length > 6;
				html += '<div class="messagecontainer note '+(moreWarnings?'minimal':'')+'" id="warnings.'+itemNo+'"><ol start="'+startNo+'">';
				for (var warnNo = 0; warnNo < jsonFile.warnings.length; warnNo++)
				{
					var msg = jsonFile.warnings[warnNo];
					if (msg.indexOf('[HTML]')==0)
					{
						html += '<li>'+msg.replace('[HTML]', '');
					}
					else
					{
						html += '<li>'+Strings.encodeTags(msg);
					}
				}
				html += '</ol></div>';
				startNo += jsonFile.warnings.length;
				if (moreWarnings)
				{
					html += '<div class="morewarnings">'+jsonFile.warnings.length+' warnings - <span class="showmore-warnings link" data-item-no="'+itemNo+'">Show more...</span></div>';
				}
			}
			if (canImport && startNo == 1)
			{
				html += 'Import to '+Strings.encodeTags(item.itemType);
			}
			if (item.liasonComment)
			{
				html += Strings.encodeTags(item.liasonComment);
			}
			html += '</td>';
			html += '</tr>';
		}

		html += '</tbody>';
		html += '</table>';
		Doc.element('itemTable').innerHTML = html;
		
		var downloadable = document.getElementsByClassName('download-file');
		for (var fileNo = 0; fileNo < downloadable.length; fileNo++)
		{
			Events.addEventHandler(downloadable[fileNo], 'click', specimen.downloadJsonFile);
		}

		var showMoreWarnings = document.getElementsByClassName('showmore-warnings');
		for (var itemNo = 0; itemNo < showMoreWarnings.length; itemNo++)
		{
			Events.addEventHandler(showMoreWarnings[itemNo], 'click', specimen.toggleWarnings);
		}
	
		var cs = document.getElementsByClassName('case-summary');
		for (var i = 0; i < cs.length; i++)
		{
			Events.addEventHandler(cs[i], 'click', Reggie.openCaseSummaryPopupOnEvent);
		}

		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	specimen.toggleWarnings = function(event)
	{
		var itemNo = Data.get(event.currentTarget, 'item-no');
		var warningsDiv = Doc.element('warnings.'+itemNo);
		Doc.addOrRemoveClass(warningsDiv, 'minimal');
		event.currentTarget.innerHTML = warningsDiv.classList.contains('minimal') ? 'Show more...' : 'Show less...';
	}

	specimen.downloadJsonFile = function(event)
	{
		var itemNo = Data.get(event.currentTarget, 'file-no');
		var file = importItems[itemNo].jsonFile;

		var url = '../ExternalSpecimen.servlet?ID='+App.getSessionId();
		url += '&cmd=DownloadJsonFile';
		url += '&file='+encodeURIComponent(file.name);
		
		Dialogs.openPopup(url, file.name, 800, 800);
	}

	specimen.validateStep1 = function(event)
	{
		Wizard.hideGoNextConfirmation();
		var frm = document.forms['reggie'];
		for (var itemNo = 0; itemNo < importItems.length; itemNo++)
		{
			var chk = frm['import.'+itemNo];
			if (chk && chk.checked) return;
		}
		Wizard.showGoNextConfirmation(false, 'Please select at least one item!');
		event.preventDefault();
	}

	// --- Step 2 -----------------------------------
	specimen.initializeStep2 = function()
	{
		Wizard.setCurrentStep(2);
		
		// Check debug by default if debug is set or not on a secure server (=production server)
		var frm = document.forms['reggie'];
		frm.debug.checked = debug || location.protocol != 'https:';

		Reggie.loadProtocols('MERGE_PROTOCOL', 'mergeProtocol', 'PIPELINE,MERGE_TYPE', 'RNAseq,Import');
		Reggie.loadSoftware('MERGE_SOFTWARE', 'mergeSoftware', 'PIPELINE,MERGE_TYPE,PARAMETER_SET', 'RNAseq,Import');

		// Load open grid clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, specimen.clustersLoaded);

		Doc.show('goregister');
		Doc.show('gocancel');
	}
	
	specimen.validateStep2 = function(event)
	{
		if (!autoAnalyzeIsValid) event.preventDefault();
	}
	
	specimen.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		frm.clusters.length = 0;
		var clusters = response.hosts;
		if (clusters.length == 0)
		{
			Doc.element('autoAnalyzeNo').checked = true;
			autoAnalyzeIsValid = true;
			Wizard.setInputStatus('autoAnalyze', 'warning', 'No Open Grid cluster available');
			return;
		}
		
		Doc.element('autoAnalyzeNo').disabled = false;
		Doc.element('autoAnalyzeYes').disabled = false;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.connection.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
		}
		specimen.autoAnalyzeOnChange();
	}
	
	specimen.autoAnalyzeOnChange = function()
	{
		Wizard.setInputStatus('autoAnalyze');
		autoAnalyzeIsValid = false;
		
		var frm = document.forms['reggie'];
		var yesChecked = Doc.element('autoAnalyzeYes').checked;
		var noChecked = Doc.element('autoAnalyzeNo').checked;
		
		frm.mergeProtocol.disabled = !yesChecked;
		frm.mergeSoftware.disabled = !yesChecked;
		frm.clusters.disabled = !yesChecked;
		frm.priority.disabled = !yesChecked;
		frm.autoConfirm.disabled = !yesChecked;
		frm.debug.disabled = !yesChecked;

		if (!yesChecked && !noChecked)
		{
			Wizard.setInputStatus('autoAnalyze', 'invalid', 'Must select Yes or No');
			return;
		}
		
		autoAnalyzeIsValid = true;
		Wizard.setInputStatus('autoAnalyze', 'valid');
		
		if (yesChecked) specimen.clusterOnChange();
	}

	
	specimen.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		if (cluster.priorities && cluster.priorities.length > 0)
		{
			frm.priority.length = 0;
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default'], p['default']);
			}
			Doc.show('job-priority');
		}
		else
		{
			Doc.hide('job-priority');
		}
	}

	/**
		Sort by file date of JSON files. Items without a file are sorted last by registration date and name.
	*/
	specimen.sortByFileDate = function(a, b)
	{
		var fa = a.jsonFile;
		var fb = b.jsonFile;
		
		if (fa && fb)
		{
			if (fa.lastModified != fb.lastModified) 
			{
				return fa.lastModified < fb.lastModified ? -1 : 1;
			}
		}
		else if (fa)
		{
			return -1;
		}
		else if (fb)
		{
			return 1;
		}
		return specimen.sortBySamplingDate(a, b);
	}

	// Sort entries by sampling date or registered date
	specimen.sortBySamplingDate = function(a, b)
	{
		var ra = a.samplingDate || a.registered;
		var rb = b.samplingDate || b.registered;
		if (ra && rb) 
		{
			if (ra != rb) return ra < rb ? -1 : 1;
		}
		else if (ra)
		{
			return -1;
		}
		else if (rb)
		{
			return 1;
		}
		return specimen.sortByName(a, b);
	}

	// Sort entries by name
	specimen.sortByName = function(a, b)
	{
		return a.name == b.name ? 0 : (a.name < b.name ? -1 : 1);
	}

	specimen.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		submitInfo.operator = frm.operator.value;
		
		var items = [];
		submitInfo.items = items;
		for (var itemNo = 0; itemNo < importItems.length; itemNo++)
		{
			var chk = frm['import.'+itemNo];
			if (chk && chk.checked)
			{
				var item = importItems[itemNo]
				var tmp = {};
				tmp.id = item.id;
				tmp.jsonFile = item.jsonFile.name;
				items[items.length] = tmp;
			}
		}

		if (Doc.element('autoAnalyzeYes').checked)
		{
			var autoAnalyze = {};
			autoAnalyze.mergeSoftware = parseInt(frm.mergeSoftware.value);
			autoAnalyze.mergeProtocol = parseInt(frm.mergeProtocol.value);
			autoAnalyze.cluster = frm.clusters.value;
			autoAnalyze.priority = parseInt(frm.priority.value);
			autoAnalyze.debug = frm.debug.checked;
			autoAnalyze.autoConfirm = frm.autoConfirm.checked;
			submitInfo.autoAnalyze = autoAnalyze;
		}

		var url = '../ExternalSpecimen.servlet?ID='+App.getSessionId();
		url += '&cmd=ImportFromJSON';
		
		Wizard.showLoadingAnimation('Importing external specimen...');
		Wizard.asyncJsonRequest(url, specimen.submissionResults, 'POST', JSON.stringify(submitInfo));
	}

	specimen.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return specimen;
}();

Doc.onLoad(ImportSpecimen.initPage);