var Blood = function()
{
	var blood = {};
	var debug = 0;
	
	var patientInfo = null;
	var bloodInfo = null;
	var siteInfo = null;
	var consentInfo = null;

	var caseIsValid = false;
	var pnrIsTested = false;
	var pnrIsValid = false;
	var pnrIsAcceptable = false;
	
	var samplingDateIsValid = true;
	var freezerDateIsValid = true;
	
	blood.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', blood.validateStep1);
		Events.addEventHandler('caseName', 'keypress', Wizard.focusOnEnter);
		Events.addEventHandler('caseName', 'blur', blood.caseNameOnChange);
		Events.addEventHandler('personalNumber', 'keyup', blood.personalNumberOnChange);
		Events.addEventHandler('personalNumber', 'keypress', Wizard.goNextOnTabOrEnter);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', blood.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', blood.validateStep2);
		Events.addEventHandler('familyName', 'blur', blood.nameOnChange);
		Events.addEventHandler('allFirstNames', 'blur', blood.nameOnChange);
		Events.addEventHandler('allFirstNames', 'keypress', Wizard.goNextOnTabOrEnter);
		
		// Step 3
		Events.addEventHandler('step-3', 'wizard-initialize', blood.initializeStep3);
		Events.addEventHandler('step-3', 'wizard-validate', blood.validateStep3);
		Events.addEventHandler('samplingDate', 'blur', blood.samplingDateTimeOnChange);
		Events.addEventHandler('samplingTime', 'blur', blood.samplingDateTimeOnChange);
		Events.addEventHandler('freezerDate', 'blur', blood.freezerDateTimeOnChange);
		Events.addEventHandler('freezerTime', 'blur', blood.freezerDateTimeOnChange);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		Buttons.addClickHandler('goupdate', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', blood.submit);
		
		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	blood.caseNameOnChange = function()
	{
		var frm = document.forms['reggie'];
		Wizard.setInputStatus('caseName');
		var caseName = frm.caseName.value.toUpperCase();
		caseIsValid = false;
		if (caseName == '')
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Missing');
			frm.caseName.focus();
			return;
		}
		if (!Reggie.checkValidCaseName(caseName, 'K?', 'C?')) // Allow 'K'-prefix and/or 'C' suffix to be used
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Expected 7-digits + optional \'K\' prefix or \'C\' suffix.');
			frm.caseName.focus();
			return;
		}

		frm.caseName.value = caseName;
		Wizard.setInputStatus('caseName', 'valid');
		caseIsValid = true;
		
		var url = '../BloodRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetBloodInfo&caseName=' + encodeURIComponent(caseName);
		Wizard.showLoadingAnimation('Loading information about case...');
		Wizard.asyncJsonRequest(url, blood.onBloodInfoLoaded);
	}

	
	blood.onBloodInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		
		// Get biosource information from the AJAX response
		bloodInfo = response.bloodInfo;
		patientInfo = response.patientInfo;
		consentInfo = response.consentInfo;
		siteInfo = response.siteInfo;
		
		if (patientInfo)
		{
			frm.personalNumber.value = patientInfo.personalNumber;
			blood.personalNumberOnChange();
			pnrIsValid = true;
			Wizard.goNext(false);
		}
		
	}

	blood.personalNumberOnChange = function()
	{
		var frm = document.forms['reggie'];
		var pnr = frm.personalNumber.value;
		
		pnrIsValid = false;
		pnrIsAcceptable = false;
		pnrIsTested = true;
		
		var pnrLen = pnr.length;
		if (pnrLen == 0)
		{
			Wizard.setInputStatus('personalNumber', 'invalid', 'A personal number is required');
			return;
		}
		
		pnrIsAcceptable = true;
		if (pnrLen != 12 && pnrLen != 10)
		{
			Wizard.setInputStatus('personalNumber', 'warning', pnrLen < 12 ? 'Too short' : 'Too long');
			return;
		}

		if (!pnr.match(/^\d+$/))
		{
			Wizard.setInputStatus('personalNumber', 'warning', 'Non-standard personal number');
			return;
		}
			
		var datePart = pnr.substring(0, pnrLen - 4); // Remove last four digits
		var dateFormat = pnrLen == 10 ? 'yyMMdd' : 'yyyyMMdd';
		var isValidDate = Dates.isDate(datePart, dateFormat);
		if (!isValidDate)
		{
			var dayInMonth = parseInt(datePart.substring(-2), 10); // Last two characters
			if (dayInMonth > 60)
			{
				// 'Samordningsnummer' has day-in-month + 60; check this
				dayInMonth -= 60;
				var tmpPnr = pnr.substring(0, pnrLen - 6); // First 4 or 6 digits is year+month
				if (dayInMonth < 10) tmpPnr += '0';
				tmpPnr += dayInMonth;
				isValidDate = Dates.isDate(tmpPnr, dateFormat);
			}
		}
		if (!isValidDate)
		{
			Wizard.setInputStatus('personalNumber', 'warning', 'Not a valid date');
			return;
		}
			
		if (!Reggie.personalNumberControlDigitCheck(pnr.substr(pnrLen == 10 ? 0 : 2)))
		{
			Wizard.setInputStatus('personalNumber', 'warning', 'Invalid control digit');
			return;
		}
			
		Wizard.setInputStatus('personalNumber', 'valid');
		pnrIsValid = true;
		Wizard.hideGoNextConfirmation();
	}

	blood.validateStep1 = function(event)
	{
		if (!pnrIsTested) persinfo.personalNumberOnChange();
		if (!caseIsValid || !pnrIsValid) 
		{
			if (!pnrIsAcceptable)
			{
				event.preventDefault();
				return;
			}
			var frm = document.forms['reggie'];
			if (!frm.verifyGoNext)
			{
				frm.personalNumber.focus();
				Wizard.showGoNextConfirmation(true, 'Check to verify registration of non-standard personal number.');
				event.preventDefault();
			}
		}
	}
	
	blood.initializeStep2 = function(event)
	{
		if (!patientInfo)
		{
			var frm = document.forms['reggie'];
			var pnr = frm.personalNumber.value;
			if (pnr.length == 10 && pnrIsValid) 
			{
				var year = Reggie.fullYear(parseInt(pnr.substr(0, 2), 10));
				pnr = year + pnr.substr(2);
			}

			var url = '../PersonalRegistration.servlet?ID='+App.getSessionId();
			url += '&cmd=GetPatientInfo';
			url += '&personalNumber=' + encodeURIComponent(pnr);
			url += '&pnrIsValid=' + pnrIsValid;

			Wizard.showLoadingAnimation('Loading patient information...');
			Wizard.asyncJsonRequest(url, blood.patientInfoLoaded);
		}
		else
		{
			blood.patientInfoLoaded();
		}
	}
	
	blood.patientInfoLoaded = function(response)
	{
		Wizard.setCurrentStep(2);
		if (response) patientInfo = response.patientInfo;
		var frm = document.forms['reggie'];
		
		if (!patientInfo.id)
		{
			Doc.show('new-patient');
			frm.patientCode.value = patientInfo.name;
			Doc.element('new.dateOfBirth').innerHTML = patientInfo.dateOfBirth ? patientInfo.dateOfBirth : "---";
			Doc.element('new.gender').innerHTML = patientInfo.gender ? patientInfo.gender : "---";
			frm.familyName.focus();
			if (!pnrIsValid)
			{
				Wizard.setInputStatus('gender', 'warning', 'Gender and DateOfBirth can\'t be automatically generated for non-standard Personal number');
			}
			Doc.show('gonext');
			Doc.show('gocancel');
		}
		else
		{
			Doc.show('existing-patient');
			Doc.element('step-2-title').innerHTML = 'Existing patient: Verify names';
			Doc.element('existing.patientCode').innerHTML = Strings.encodeTags(patientInfo.name);
			Doc.element('existing.familyName').innerHTML = Strings.encodeTags(patientInfo.familyName);
			Doc.element('existing.allFirstNames').innerHTML = Strings.encodeTags(patientInfo.allFirstNames);
			if (patientInfo.dateOfBirth) Doc.element('existing.dateOfBirth').innerHTML = Strings.encodeTags(patientInfo.dateOfBirth);
			if (patientInfo.gender) Doc.element('existing.gender').innerHTML = Strings.encodeTags(patientInfo.gender);
			Wizard.goNext(true);	
		}
	}
	
	blood.validateStep2 = function(event)
	{
		var valid = true;
		var acceptable = true;
		var frm = document.forms['reggie'];

		// New patient only
		if (!patientInfo.id)
		{
			// Validate 'New patient' form
			if (frm.allFirstNames.value == '')
			{
				Wizard.setInputStatus('allFirstNames', 'warning', 'Missing');
				valid = false;
			}
			else
			{
				Wizard.setInputStatus('allFirstNames', 'valid');
			}
			
			if (frm.familyName.value == '')
			{
				Wizard.setInputStatus('familyName', 'warning', 'Missing');
				valid = false;
			}
			else
			{
				Wizard.setInputStatus('familyName', 'valid');
			}
			if (frm.patientCode.value == '')
			{
				Wizard.setInputStatus('patientCode', 'invalid', 'Missing');
				frm.patientCode.focus();
				valid = false;
				acceptable = false;
			}
			else
			{
				Wizard.setInputStatus('patientCode', 'valid');
				patientInfo.name = frm.patientCode.value;
			}
			
			if (event.detail.auto)
			{
				if (!valid)
				{
					Wizard.showGoNextConfirmation(true, 'Check to verify registration with missing name(s).');
					event.preventDefault();
				}
			}
			else
			{
				if (!acceptable) event.preventDefault();
			}
		}
	}
	
	blood.nameOnChange = function(event)
	{
		var field = event.target;
		patientInfo[field.name] = field.value;
		if (field.value == '')
		{
			Wizard.setInputStatus(field, 'warning', 'Missing');
		}
		else
		{
			Wizard.setInputStatus(field, 'valid');
			Wizard.hideGoNextConfirmation();
		}
	}
	
	
	blood.initializeStep3 = function(event)
	{
		var updateMode = bloodInfo.id;
		var frm = document.forms['reggie'];

		if (updateMode)
		{
			Doc.element('step-3-title').innerHTML = 'Update blood \''+Strings.encodeTags(bloodInfo.name)+'\'';
			if (bloodInfo.samplingDate != null)
			{
				frm.samplingDate.value = bloodInfo.samplingDate.substring(0, 8);
				frm.samplingTime.value = bloodInfo.samplingDate.substring(9, 13);
				blood.samplingDateTimeOnChange();
			}
			if (bloodInfo.freezerDate != null)
			{
				frm.freezerDate.value = bloodInfo.freezerDate.substring(0, 8);
				frm.freezerTime.value = bloodInfo.freezerDate.substring(9, 13);
				blood.freezerDateTimeOnChange();
			}
			frm.serum.checked = bloodInfo.serum == 'Yes';
			if (bloodInfo.bloodSample)
			{
				Forms.selectListOption(frm.bloodSample, bloodInfo.bloodSample);
			}
			if (bloodInfo.otherPathNote)
			{
				frm.otherPathNote.value = bloodInfo.otherPathNote;
			}
		}
		else
		{
			// Select 'PreNeo' if case name has 'C' suffix
			var caseName = frm.caseName.value;
			var cLength = caseName.length;
			if (cLength > 7 && caseName.substr(cLength-1, 1) == 'C')
			{
				Forms.selectListOption(frm.bloodSample, 'PreNeo');
			}
			if (consentInfo && consentInfo.consent)
			{
				frm.copyConsent.value = consentInfo.id;
				var html = "From '" + Strings.encodeTags(consentInfo.name) + "' (" + consentInfo.consent;
				if (consentInfo.consentDate) html += '; ' + consentInfo.consentDate;
				html += ')';
				Doc.element('copyConsent.text').innerHTML = html;
				Doc.show("copyConsentSection");
			}
		}
		
		// Special case for Uppsala and Jönköping
		if (siteInfo && (siteInfo.prefix == '88' || siteInfo.prefix == '99'))
		{
			// Plasm/serum check box should not be selected
			frm.serum.checked = false;
		}
		
		if (updateMode)
		{
			Doc.show('goupdate');
		}
		else
		{
			Doc.show('goregister');
		}
		Doc.show('gocancel');

		Wizard.setCurrentStep(3);
		frm.samplingDate.focus();
	}
	
	blood.validateStep3 = function(event)
	{
		if (!samplingDateIsValid || !freezerDateIsValid)
		{
			event.preventDefault();
		}
	}

	blood.samplingDateTimeOnChange = function()
	{
		var frm = document.forms['reggie'];
		samplingDateIsValid = false;
		Wizard.setInputStatus('samplingDate');
		
		var samplingDate = frm.samplingDate.value;
		var samplingTime = frm.samplingTime.value;
		
		var isUppsala = siteInfo && siteInfo.prefix == '88';
		
		if (samplingDate != '' || samplingTime != '')
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			samplingDate = Reggie.autoFillDate(samplingDate, true);
			frm.samplingDate.value = samplingDate;

			samplingTime = Reggie.autoFillTime(samplingTime);
			// Set sampling time to 0000 for Uppsala since it is not known
			if (!samplingTime && isUppsala) samplingTime = '0000';
			frm.samplingTime.value = samplingTime;
			
			if (!Dates.isDate(samplingDate, 'yyyyMMdd'))
			{
				Wizard.setInputStatus('samplingDate', 'invalid', 'Not a valid date');
				return;
			}
			
			// Automatically fill in freezer date except for Uppsala since it is not known
			if (frm.freezerDate.value == '' && !isUppsala)
			{
				frm.freezerDate.value = samplingDate;
			}
			
			if (!Dates.isDate(samplingDate + ' ' + samplingTime, 'yyyyMMdd HHmm'))
			{
				if (samplingDate != '') 
				{
					Wizard.setInputStatus('samplingDate', 'invalid', 'Not a valid time');
				}
				return;
			}
			Wizard.setInputStatus('samplingDate', 'valid');
		}
		samplingDateIsValid = true;
	}

	blood.freezerDateTimeOnChange = function()
	{
		var frm = document.forms['reggie'];
		freezerDateIsValid = false;
		Wizard.setInputStatus('freezerDate');
		
		var freezerDate = frm.freezerDate.value;
		var freezerTime = frm.freezerTime.value;
		var freezerTimestamp;
		
		if (freezerDate != '' || freezerTime != '')
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			freezerDate = Reggie.autoFillDate(freezerDate, true);
			frm.freezerDate.value = freezerDate;

			freezerTime = Reggie.autoFillTime(freezerTime);
			frm.freezerTime.value = freezerTime;
			
			if (!Dates.isDate(freezerDate, 'yyyyMMdd'))
			{
				Wizard.setInputStatus('freezerDate', 'invalid', 'Not a valid date');
				return;
			}
			freezerTimestamp = Dates.parseString(freezerDate + ' ' + freezerTime, 'yyyyMMdd HHmm');
			if (freezerTimestamp == null)
			{
				if (freezerDate != '') 
				{
					Wizard.setInputStatus('freezerDate', 'invalid', 'Not a valid time');
				}
				return;
			}
			
			Wizard.setInputStatus('freezerDate', 'valid');
		}
		
		freezerDateIsValid = true;

		if (samplingDateIsValid && freezerTimestamp)
		{
			var samplingTimestamp = Dates.parseString(frm.samplingDate.value + ' ' + frm.samplingTime.value, 'yyyyMMdd Hmm');
			if (samplingTimestamp && samplingTimestamp.getDate() != freezerTimestamp.getDate())
			{
				Wizard.setInputStatus('freezerDate', 'warning', 'Sampling and freezer dates are different');
			}
		}
	}

	blood.submit = function()
	{
		var frm = document.forms['reggie'];

		bloodInfo.samplingDate = Strings.trim(frm.samplingDate.value + ' ' + frm.samplingTime.value);
		bloodInfo.freezerDate = Strings.trim(frm.freezerDate.value + ' ' + frm.freezerTime.value);
		bloodInfo.serum = frm.serum.checked ? "Yes" : "No";
		bloodInfo.bloodSample = frm.bloodSample.value;
		bloodInfo.otherPathNote = frm.otherPathNote.value;
		if (frm.copyConsent.checked && frm.copyConsent.value)
		{
			bloodInfo.copyConsent = parseInt(frm.copyConsent.value, 10);
		}

		var updateMode = bloodInfo.id;
		var submitInfo = {};
		submitInfo.patientInfo = patientInfo;
		submitInfo.bloodInfo = bloodInfo;

		var url = '../BloodRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=' + (updateMode ? 'UpdateBlood' : 'CreateBlood');
		
		Wizard.showLoadingAnimation('Registering blood information...');
		Wizard.asyncJsonRequest(url, blood.submissionResults, 'POST', JSON.stringify(submitInfo));

	}
	
	blood.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	
	return blood;
}();

Doc.onLoad(Blood.initPage);

